import sys
import toml
import json

from suai_observer.config import Config
from suai_observer.session import SuaiSession

def config_from_file(filename: str) -> Config:
	content = None
	with open(filename, 'r') as f:
		content = toml.load(f)

	timeout = content["timeout"]
	assert type(timeout) == int

	cookiefile = content["cookiefile"]
	assert type(cookiefile) == str

	username = content["username"]
	assert type(username) == str

	password = content["password"]
	assert type(password) == str

	return Config(timeout, cookiefile, username, password)

def main():
	configfile = sys.argv[1]
	config = config_from_file(configfile)

	session = SuaiSession(config.timeout, config.cookiefile, config.username, config.password)

	comma = ''
	sys.stdout.write('[')
	for report in session.reports():
		sys.stdout.write(comma)

		json_report = {
			"id": report.id,
			"task": report.task,
			"professor": report.professor,
			"status": report.status,
			"score": report.score,
			"date": report.date,
		}
		json.dump(json_report, sys.stdout, ensure_ascii=False)

		comma = ','
	sys.stdout.write(']')



import pickle
import requests
import bs4

from suai_observer.report import ReportsFromPage

from bs4 import BeautifulSoup, SoupStrainer
from typing import Callable
from requests import Response

class SuaiSession:
	def __init__(self, timeout: int, cookiefile: str, username: str, password: str):
		self.timeout = timeout
		self.session = requests.Session()
		self.cookiefile = cookiefile
		try:
			with open(cookiefile, 'rb') as f:
				cookies = pickle.load(f)
				self.session.cookies.update(cookies)
		except:
			pass

		self.username = username
		self.password = password

	@staticmethod
	def login_redirected(response: Response) -> bool:
		return response.url.find("https://sso.guap.ru") != -1

	@staticmethod
	def form_url(login_page: str) -> str:
		only_form = SoupStrainer(name="form", attrs={'id': "kc-form-login"})
		soup = BeautifulSoup(login_page, parse_only=only_form, features="html.parser")
		url = soup.form.attrs['action']
		return url

	def update_login_cookies(self, form_url: str):
		form_data = {
			"username": self.username,
			"password": self.password,
			"credentialId": '',
		}
		self.session.post(form_url, data=form_data, timeout=self.timeout).raise_for_status()
		self.save_cookies()

	def save_cookies(self):
		with open(self.cookiefile, 'wb') as f:
			pickle.dump(self.session.cookies, f)

	def do_while_login(self, f: Callable[[requests.Session], Response]) -> Response:
		response = f(self.session)
		login_try_timeout = 0
		while SuaiSession.login_redirected(response):
			if login_try_timeout >= 5:
				raise TimeoutError("Way too much login attempts")
			login_try_timeout += 1

			self.update_login_cookies(SuaiSession.form_url(response.text))
			response = f(self.session)
		return response

	def get(self, *args, **kwargs) -> Response:
		return self.do_while_login(lambda s: s.get(timeout=self.timeout, *args, **kwargs))

	def post(self, *args, **kwargs) -> Response:
		return self.do_while_login(lambda s: s.post(timeout=self.timeout, *args, **kwargs))

	def reports(self) -> ReportsFromPage:
		response = self.get("https://pro.guap.ru/inside/student/reports?perPage=999999999&page=1")
		response.raise_for_status()
		return ReportsFromPage(response.text)



class Config:
	def __init__(self, timeout: int, cookiefile: str, username: str, password: str):
		self.timeout = timeout
		self.cookiefile = cookiefile
		self.username = username
		self.password = password



import re
from bs4 import BeautifulSoup, SoupStrainer

class Report:
	def __init__(self, id: int, task: str, professor: str, status: str, score: str, date: str):
		self.id = id
		self.task = task
		self.professor = professor
		self.status = status
		self.score = score
		self.date = date

class ReportsFromPage:
	def __init__(self, page: str):
		only_tr = SoupStrainer(name="tr")
		soup = BeautifulSoup(page, parse_only=only_tr, features="html.parser")
		self.tr = soup.tr.next_sibling

	def __iter__(self):
		return self

	REPORT_ID_RE = re.compile(r'/inside/student/reports/([0-9]+)/download')

	def __next__(self):
		if self.tr is None:
			raise StopIteration()

		[link, _, task, professor, status, score, date] = self.tr.find_all('td')

		link = link.find(name='a', attrs={'title': 'Скачать отчет'}).attrs['href']
		task = task.find('a').text.strip()
		professor = professor.find('a').text.strip()
		status = status.find('div').find('span').text.strip()
		score = score.find('div').find('span').text.strip()
		date = date.find('div').find('span').text.strip()

		id = int(ReportsFromPage.REPORT_ID_RE.match(link).group(1))

		self.tr = self.tr.next_sibling

		return Report(id, task, professor, status, score, date)
